<?PHP
//Abstract Pool Project Class .1
//This implements a generic pool project class that should be extended for each pool subtype
//Requires the db_init and error_init files

require_once('db_init.php');

abstract class Pool_Project{
	//Public variables (from db)
	public $id = false;				//ID of the project
	public $hr_id = '';				//Human readable ID of the project
	public $title = '';				//Title of the project
	public $description = '';		//Description of the project
	public $pool_type = '';			//Pool this project exists in
	public $intent_num = 0;			//Number of intents in this project
	public $approach_num = 0;		//Number of approaches in this project
	public $release_num = 0;		//Number of releases in this project
	public $last_modified = '';		//Datetime when any artifact in the project last changed
	public $last_modified_unix = '';//Unix timestamp when the project was last modified
	public $rating = 0;				//Rating of the project
	public $basis = 0;				//Basis of the project
	public $subratings = array();	//Ratings for this project, broken down by category
	public $contributors = array();	//Contributors to this project
	public $subjects = array();		//Subjects for this project
	public $reviews = array();		//Reviews for this project
	public $relationships = array();//Relationships for this project
	public $thumb_id = false;		//ID of the project's thumbnail image in the thumbs table
	
	//Public variables (set by pool type)
	//These are set when the [pool_type]_pool.ini file is read, only here for reference
	//Children should always call Pool_Project->read_ini before anything else
	//See Pool_Project->read_ini below
//	public $has_artifacts = false;		//Does this project have any artifacts?
	//These are only created if has_artifacts is true
//	public $stages = array();			//Array of valid stages for projects of type [pool_type]
// 	public $intent_aspects = array();	//Array of valid aspects for intents of type [pool_type]
// 	public $approach_aspects = array();	//Array of valid aspects for approaches of type [pool_type]
// 	public $release_aspects = array();	//Array of valid aspects for releases of type [pool_type]
//	public $project_ratings = array();	//Array of valid ratings for projects of type [pool_type]
//	public $intent_ratings = array();	//Array of valid ratings for intents of type [pool_type]
//	public $approach_ratings = array();	//Array of valid ratings for approaches of type [pool_type]
//	public $release_ratings = array();	//Array of valid ratings for releases of type [pool_type]
//	public $intents = array();		//Array of intents in this project.  This should contain only
//									//objects of type [pool]_Intent.
//	public $approaches = array();	//Array of approaches in this project.  This should contain only
//									//objects of type [pool]_Approach.
//	public $releases = array();		//Array of releases in this project.  This should contain only
//									//objects of type [pool]_Release.
									
	//Protected db object
	//This is filled in the constructor
	protected $data = '';
					
	//Abstract functions				
	//abstract public function new_project();			//Creates a new project
	//abstract public function new_intent();			//Creates a new intent
	//abstract public function new_approach();		//Creates a new approach
	//abstract public function new_release();			//Creates a new release
	
	//Generic functions
	public function __construct($construct_var){
		//Generic constructor
		//$construct_var can be either a string or an array
		//If $construct_var is a string, it is assumed that you are loading a project from the db
		//If $construct_var is an array, it is assumed you are building a new project
		//Acutal implimentation of either is left up to the child class
		
		//Set up the db object
		$this->data = &$GLOBALS['data'];
		
		//Read the ini file.  $this->ini_file should be set in the child class
		$this->read_ini(POOL_INI_PATH.$this->ini_file);
		
		if(is_array($construct_var)) $this->new_project($construct_var);
		else{
			//If $construct_var is not numeric only, it is an hr_id.  Get the numeric id.
			if(!is_numeric($construct_var)){
				$this->id = $this->resolve_hr($construct_var, 'project');
				if($this->id == false) return false;
			}
			else $this->id = $construct_var;
		}
	}
	
	public function load_all(){
		//Loads all project stages from the db
		if($this->has_artifacts){
			$this->load_project();
			$this->load_artifacts('intent');
			$this->load_artifacts('approach');
			$this->load_artifacts('release');
			$this->load_reviews();
			$this->load_relations();
		} else {
			$this->load_project();
			if(!$this->load_reviews()) return false;
		}
	}
	
	public function load_project(){
		//Loads data from the project table
		//Project data should all be generic, so this can go in the abstract class
		//Make sure the id has been set somehow
		if(!$this->check_id('load_project')) return false;
		
		//Get the project row
		$q = "SELECT *, UNIX_TIMESTAMP(`last_modified`) AS last_modified_unix FROM `project` WHERE `id` = ?";
		$result = $this->data->getRow($q, array($this->id));
		if(!$this->check_db($result, 'load_project')) return false;
		foreach($result as $key=>$col) {
			$this->$key = $col;
		}
		if(!$this->load_contributors()) return false;
		if(!$this->load_subjects()) return false;
		if(!$this->load_subratings()) return false;
		return true;
	}
	
	public function load_contributors(){
		//Loads all contributors from all levels of a project
		//First, see if the project has artifacts
		//If so, use project_artifact as a linking table
		//If not, just go straight to person_artifact with the project id
		if(!$this->check_id('load_contributors')) return false;
		
		if($this->has_artifacts){
			$q = 'SELECT DISTINCT `person`.`id`, `person`.`written`, GROUP_CONCAT(DISTINCT `person_artifact`.`role` SEPARATOR ",") AS roles, COUNT(DISTINCT `project_artifact`.`artifact_id`) as versions
					FROM `person`, `person_artifact`, `project_artifact`
					WHERE `project_artifact`.`project_id` = ?
						AND `project_artifact`.`artifact_type` = `person_artifact`.`artifact_type`
						AND `person_artifact`.`artifact_id` = `project_artifact`.`artifact_id` 
						AND `person_artifact`.`person_id` = `person`.`id`
					GROUP BY `person_artifact`.`person_id`
					ORDER BY `versions` DESC';
		} else {
			$q = "SELECT `person`.`id` AS id, `person`.`written` AS written, 
						GROUP_CONCAT(DISTINCT `person_artifact`.`role`) AS roles, COUNT(DISTINCT `project_artifact`.`artifact_id`) as versions
					FROM `person`, `person_artifact`
					WHERE `person_artifact`.`artifact_id` = ? AND
						`person_artifact`.`artifact_type` = 'project' AND
						`person_artifact`.`person_id` = `person`.`id`
					GROUP BY `person_artifact`.`person_id`
					ORDER BY `versions` DESC";	
		}
		$result = $this->data->getAll($q, array($this->id));
		if(!$this->check_db($result, 'load_contributors')) return false;
		$this->contributors = $result;
		return true;	
	}
	
	public function load_subjects(){
		//Loads all subjects for this project
		if(!$this->check_id('load_subjects')) return false;
		
		$q = "SELECT `subject`.`id` AS id, `subject`.`written` AS written
				FROM `subject`, `project_subject`
				WHERE `project_subject`.`project_id` = ? AND
					`project_subject`.`subject_id` = `subject`.`id`";
		$result = $this->data->getAll($q, array($this->id));
		if(!$this->check_db($result, 'load_subject')) return false;
		$subjects = array();
		foreach($result as $subject) $subjects[$subject['id']] = $subject['written'];
		$this->subjects = $subjects;
		return true;
	}
	
	public function load_reviews(){
		//Loads all reviews for this project as review objects
		//If the project has artifacts, the reviews of each artifact are loaded into $this->reviews
		//If there are no artifacts, load the project-level reviews directly
		if(!$this->check_id('load_reviews')) return false;
		if($this->has_artifacts == false){
			$q = "SELECT `id` FROM `review` WHERE `artifact_id` = ? AND `artifact_type` = 'project'";
			$result = $this->data->getAll($q, array($this->id));
			if(!$this->check_db($result, 'load_reviews')) return false;
			foreach($result as $project_review) $this->reviews[] = new Review($project_review['id']);
			return true;
		} else {
			//if this project has artifacts, load the reviews of the individual artifacts to the project level
			foreach($this->intent as $intent){
				foreach($intent->reviews as $key=>$review) $this->reviews[$review->id] = $review;
			}
			foreach($this->approach as $approach){
				foreach($approach->reviews as $key=>$review) $this->reviews[$review->id] = $review;
			}
			foreach($this->release as $release){
				foreach($release->reviews as $key=>$review) {
					$this->reviews[$review->id] = $review;
				}
			}			
		}
	}
	
	public function load_subratings(){
		//Loads subratings for this project
		if(!$this->check_id('load_subratings')) return false;
		
		$q = "SELECT `category`, `rating`, `basis` FROM `artifact_ratings` WHERE `artifact_id` = ? AND `artifact_type` = 'project'";
		$result = $this->data->getAll($q, array($this->id));
		if(!$this->check_db($result, 'load_subratings')) return false;
		foreach($result as $subrating) $this->subratings[$subrating['category']] = array('rating' => $subrating['rating'], 'basis' => $subrating['basis']);
		return true;
	}
	
	public function load_relations(){
		//Loads the relationships for this object
		if(!$this->check_id('load_relations')) return false;
		
		$q = 'SELECT * FROM `relations` WHERE (`subject_id` = ? AND `subject_type` = ?) OR (`predicate_id` = ? AND `predicate_type` = ?)';
		$result = $this->data->getAll($q, array($this->id, $this->pool_type, $this->id, $this->pool_type));
		if(!$this->check_db($result, 'load_relations')) return false;
		foreach($result as $relationship) $this->relationships[$relationship['id']] = $relationship;
		
		//Add titles and recorder name
		foreach($this->relationships as &$relationship){
			if(strpos($relationship['subject_type'], 'ref') === false) $q = 'SELECT `title` FROM `project` WHERE `id` = ?';
			else $q = 'SELECT `title` FROM `reference` WHERE `id` = ?';
			$relationship['subject_title'] = $this->data->getOne($q, array($relationship['subject_id']));
			
			if(strpos($relationship['predicate_type'], 'ref') === false) $q = 'SELECT `title` FROM `project` WHERE `id` = ?';
			else $q = 'SELECT `title` FROM `reference` WHERE `id` = ?';
			$relationship['predicate_title'] = $this->data->getOne($q, array($relationship['predicate_id']));			
			
			$q = 'SELECT `written` FROM `person` WHERE `id` = ?';
			$relationship['recorder_written'] = $this->data->getOne($q, array($relationship['recorder_id']));
		}

		return true;
	}
	
	public function load_artifacts($artifact_type='intent'){
		//Loads data from the artifact_type table for this project
		//Data should all be generic, so this can go in the abstract class
		//However, this depends on [artifact_type]_aspects being defined in the child
		//$artifact_type can be intent, approach, or release
		
		//Make sure the id has been set somehow and it has artifacts
		if(!$this->check_id('load_artifacts')) return false;
		if(!$this->has_artifacts) return false;
		
		//Get all the artifacts of the artifact type associated with this project id
		$q = "SELECT `$artifact_type`.id FROM `$artifact_type`, `project_artifact` 
			  WHERE `project_artifact`.`artifact_id` = `$artifact_type`.`id`
			  AND `project_artifact`.`artifact_type` = '$artifact_type'
			  AND `project_artifact`.`project_id` = ?";
		$result = $this->data->getAll($q, array($this->id));
		if(!$this->check_db($result, 'load_artifacts')) return false; 
		
		//set the class name based on the artifact type
		$class_name = "Pool_".ucfirst($artifact_type);
		
		//Store each returned artifact as a Pool_[artifact_type] object
		foreach($result as $row)
		{
			//Create a new Pool_[artifact_type] object for each artifact
			array_push($this->$artifact_type, new $class_name($row['id']));
		}
		
		return true;
	}
	
	public function write_js(){
		//writes the JS array for the entire project
		
		require_once('smarty_init.php');
		$smarty = &$GLOBALS['smarty'];
		
		$smarty->assign('project', $this);
		$smarty->left_delimiter = '<{';
		$smarty->right_delimiter = '}>';		
		$smarty->display('js_project_array.tpl');
		$smarty->left_delimiter = '{';
		$smarty->right_delimiter = '}';
	}
	
	public function write_js_short(){
		//writes the short-form JS array for the entire project
		
		require_once('smarty_init.php');
		$smarty = &$GLOBALS['smarty'];
		
		$cardinality = $this->intent_num + $this->approach_num + $this->release_num;
		
		$smarty->assign('relations', count($this->relationships));
		$smarty->assign('cardinality', $cardinality);
		$smarty->assign('project', $this);
		$smarty->left_delimiter = '<{';
		$smarty->right_delimiter = '}>';		
		$smarty->display('js_project_array_short.tpl');
		$smarty->left_delimiter = '{';
		$smarty->right_delimiter = '}';	
	}
	
	public function save_js(){
		//saves the JS array for the entire project to the database
		
		require_once('smarty_init.php');
		$smarty = &$GLOBALS['smarty'];
		
		$smarty->assign('project', $this);
		$smarty->left_delimiter = '<{';
		$smarty->right_delimiter = '}>';		
		$js_cache = $smarty->fetch('js_project_array.tpl');
		$smarty->left_delimiter = '{';
		$smarty->right_delimiter = '}';
		
		$q = 'UPDATE `project` SET `js_cache` = ? WHERE `id` = ?';
		$this->data->query($q, array($js_cache, $this->id));
	}
	
	public function save_js_short(){
		//writes the short-form JS array for the entire project
		
		require_once('smarty_init.php');
		$smarty = &$GLOBALS['smarty'];
		
		$cardinality = $this->intent_num + $this->approach_num + $this->release_num;
		$short_desc = substr($this->description, 0, 400);
		if(strlen($this->description) > 400) $short_desc = $short_desc.'...';
		
		$smarty->assign('relations', count($this->relationships));
		$smarty->assign('cardinality', $cardinality);
		$smarty->assign('short_desc', $short_desc);
		$smarty->assign('mod_date', $this->most_recently_modified());
		$smarty->assign('project', $this);
		$smarty->left_delimiter = '<{';
		$smarty->right_delimiter = '}>';		
		$js_cache = $smarty->fetch('js_project_array_short.tpl');
		$smarty->left_delimiter = '{';
		$smarty->right_delimiter = '}';	

		$q = 'UPDATE `project` SET `js_cache_short` = ? WHERE `id` = ?';
		$this->data->query($q, array($js_cache, $this->id));		
	}	
	
	public function get_reviewer_creds(){
		//get the credibilities for everybody who has written reviews of this project's artifacts
		
		$q = 'SELECT `person_ratings`.`person_id`, `person_ratings`.`category`, `person_ratings`.`rating`
				FROM `person_ratings`, `review`, `project_artifact`
				WHERE `review`.`artifact_id` = `project_artifact`.`artifact_id`
					AND `review`.`artifact_type` = `project_artifact`.`artifact_type`
					AND `review`.`recorder_id` = `person_ratings`.`person_id`
					AND `project_artifact`.`project_id` = ?
					GROUP BY `person_ratings`.`person_id`, `person_ratings`.`category`, `person_ratings`.`rating`';
		$result = $this->data->getAll($q, array($this->id));
		if(!$this->check_db($result, 'get_reviewer_creds')) return false;
		$creds = array();
		foreach($result as $cred){
			if(!isset($creds[$cred['person_id']])) $creds[$cred['person_id']] = array();
			$creds[$cred['person_id']][$cred['category']] = $cred['rating'];
		}
		
		return $creds;
	}
	
	public function set_rating_and_basis(){
		//Sets the rating and basis for this project
		//Assumes that get_contributors and get_reviews have already been run.
		
		//First get the credibilities of everybody who has reviewed the project
		$creds = $this->get_reviewer_creds();
		
		//Apply credibilities to review ratings
		//$ratings is an array of rating categories.  Each category has a weight and a total.  Weight is the sum of credibilities, while total is the sum of (credibilities * ratings)
		$ratings = array();
		foreach($this->reviews as $review){
			foreach($review->subratings as $category => $subrating){
				//Check to see if this person has a credibility for this category of rating.  If it does not exist, set it to 1
				if(!isset($creds[$review->recorder_id][$category])) $creds[$review->recorder_id][$category] = 1;
				//If this is the first rating of the category, initialize it in the ratings array
				if(!isset($ratings[$category])) $ratings[$category] = array('weight' => 0, 'total' => 0, 'basis' => 0);
				//Add the rating values
				$ratings[$category]['weight'] += $creds[$review->recorder_id][$category];
				$ratings[$category]['total'] += $creds[$review->recorder_id][$category] * $subrating['rating'];
				$ratings[$category]['basis']++;
			}
		}
		
		//Calculate the weighted ratings, as well as the overall rating and basis for the project
		$overall = $num_ratings = 0;
		foreach($ratings as $category => $subrating){
			$subrating['final'] = $subrating['total'] / $subrating['weight'];
			$overall += $subrating['final'];
			$ratings[$category] = $subrating;
			$num_ratings++;
		}
		if($num_ratings > 0) $overall = $overall / $num_ratings;
		else $overall = 0;
		$basis = count($this->reviews);

		//Store the new rating, subratings, and basis
		$q = 'UPDATE `project` SET `rating` = ?, `basis` = ? WHERE `id` = ?';
		$result = $this->data->query($q, array($overall, $basis, $this->id));
		if(!$this->check_db($result, 'set_rating_and_basis')) return false; 

		foreach($ratings as $category => $subrating){
			$values_arr = array();
			$data_arr = array();
					
			$values_arr[] = '(?, "project", ?, ?, ?, ?)';
			$data_arr[] = $this->id;
			$data_arr[] = $category;
			$data_arr[] = $subrating['final'];
			$data_arr[] = $subrating['basis'];
			$data_arr[] = $this->hr_id.'_'.$category;
			$values_str = implode(',', $values_arr);
			
			$q = "INSERT INTO `artifact_ratings` (`artifact_id`, `artifact_type`, `category`, `rating`, `basis`, `hr_link`)
					VALUES $values_str ON DUPLICATE KEY UPDATE `rating` = VALUES(`rating`), `basis` = VALUES(`basis`)";
			$result = $this->data->query($q, $data_arr);
			if(!$this->check_db($result, 'set_rating_and_basis')) return false;
		}
		//update the js_cache and js_cache_short for the project
		$this->reload_all();
		$this->save_js();
		$this->save_js_short();
		
		//update the contributors' ratings
		foreach($this->contributors as $contributor){
			$oContrib = new Pool_Person($contributor['id']);
			$oContrib->set_rating_and_basis();
		}
		
		return true;
	}
	
	public function recount_stages(){
	//Recounts the number of intents, approaches, and releases for this project and updates the project table with those numbers
		foreach($this->stages as $stage){
			$q = "UPDATE `project` SET `".$stage."_num` = ( SELECT COUNT( * ) FROM `project_artifact` WHERE `project_id` = ? AND `artifact_type` = ? ) WHERE `id` = ?";
			$result = $this->data->query($q, array($this->id, $stage, $this->id));
			if(!$this->check_db($result, 'recount_stages')) return false;
		}
	}
	
	public function reload_all(){
	//Reloads this object from the database
		$this->subratings=array();
		$this->contributors=array();
		$this->reviews=array();
		$this->subjects=array();
		if($this->has_artifacts){
			$this->intent=array();
			$this->approach=array();
			$this->release=array();
		}
		
		$this->load_all();
	}
	
	public function save_subjects($subject_array){
		//saves a set of subject ids in project_subject
		//First delete all the old saved subjects
		$q = 'DELETE FROM `project_subject` WHERE `project_id` = ?';
		$result = $this->data->query($q, array($this->id));
		if(!$this->check_db($result, 'save_subjects')) return false;
		
		//Then add the new ones
		$q = 'INSERT INTO `project_subject` (`project_id`, `subject_id`, `hr_link`) 
				VALUES (?, ?, ?) 
				ON DUPLICATE KEY UPDATE `hr_link` = `hr_link`';
		foreach($subject_array as $subject_id){
			$oSubject = new Subject($subject_id);
			$hr_link = $this->hr_id.'_'.$oSubject->hr_id;
			$result = $this->data->query($q, array($this->id, $oSubject->id, $hr_link));
			if(!$this->check_db($result, 'save_subjects')) return false;
		}
	}
	
	public function most_recently_modified(){
		//Finds the most recently modified artifact attached to this project
		
		$q = 'SELECT `artifact_id`, `artifact_type` FROM `project_artifact` WHERE `project_id` = ? AND `artifact_type` <> ?';
		$result = $this->data->getAll($q, array($this->id, 'project'));
		if(!$this->check_db($result, 'most_recently_modified')) return false;
		$all_artifacts = $result;
		$dates = array();	
		foreach($all_artifacts as $artifact){
			$q = "SELECT UNIX_TIMESTAMP(`date`) AS last_modified_unix FROM `$artifact[artifact_type]` WHERE `id` = ?";
			$result = $this->data->getOne($q, array($artifact['artifact_id']));
			if(!$this->check_db($result, 'most_recently_modified')) return false;
			array_push($dates, $result);
		}
		if(count($dates) > 0) return max($dates);
		else return 0;
	}
	
	//Protected functions
	protected function check_id($func){
		//Return true if the id has been set, false and an error otherwise
		if($this->id == false){
			trigger_error("Pool_Project->$func failed: No id defined", E_USER_WARNING);
			return false;
		} else return true;
	}
	
	protected function check_db($result, $func){
		//Return true if $result is not DB::Error, false and an error otherwise
		if(DB::isError($result)){
			trigger_error("Pool_Project->$func failed: ".$this->data->last_query." \n".$result->getMessage(), E_USER_WARNING);	
			return false;
		} else return true;
	}
	
	protected function read_ini($filename){
		//Reads a pool ini file and sets variables appropriately
		//See sample.ini for details on ini structure
		$init_array = parse_ini_file($filename, true);
		$this->pool_type = $init_array['general']['type'];
		$this->has_artifacts = $init_array['general']['has_artifacts'] == 'true' ? true : false;
		if($this->has_artifacts){
			//Determine what stages are in this pool type and set container objects for them
			$this->stages = explode(',', $init_array['general']['stages']);
			foreach($this->stages as $stage) $this->$stage = array();
			
			//Determine what aspects each stage has
			foreach($init_array['aspects'] as $stage=>$aspects){
				$stage_name = $stage.'_aspects';
				$exploded = explode(',', $aspects);
				$this->$stage_name = $exploded;
			}
			
			//Determine what ratings each stage is subject to
			foreach($init_array['ratings'] as $stage=>$ratings){
				$stage_name = $stage.'_ratings';
				$exploded = explode(',', $ratings);
				$this->$stage_name = $exploded;	
			}
		}
	}	
	
	//Static functions
	public static function resolve_hr($hr_id, $table){
		//Turns a human readable ID into a numeric one
		$data = &$GLOBALS['data'];
		$q = "SELECT `id` FROM `$table` WHERE `$table`.`hr_id` = ?";
		$result = $data->getOne($q, array($hr_id));
		if(DB::isError($result)){
			trigger_error("Could not resolve hr_id '$hr_id' in '$table' table.", E_USER_WARNING);
			return false;
		} else {
			return $result;
		}
	}
	
	public static function id_to_hr($id, $table){
		//Turns a numeric id into a human readable one
		$data = &$GLOBALS['data'];
		$q = "SELECT `hr_id` FROM `$table` WHERE `$table`.`id` = ?";
		$result = $data->getOne($q, array($id));
		if(DB::isError($result)){
			trigger_error("Could not resolve hr_id '$hr_id' in '$table' table.", E_USER_WARNING);
			return false;
		} else {
			return $result;
		}		
	}
	
	public static function resolve_artifact_id($artifact_type, $artifact_id){
		//Turns the artifact ID of artifactType into the projectID it is a part of
		$data = &$GLOBALS['data'];
		$q = 'SELECT `project_artifact`.`project_id`
				FROM `project_artifact`
				WHERE `project_artifact`.`artifact_type` = ?
					AND `project_artifact`.`artifact_id` = ?';
		$result = $data->getOne($q, array($artifact_type, $artifact_id));
		//print_r($data->last_query);
		if(DB::isError($result)){
			trigger_error("Could not get project ID for $artifact_type ID=$artifact_id.", E_USER_WARNING);
			return false;
		} else return $result;
	}
	
	public static function get_cached_js($project_id){
		//Gets the cached javascript array for $project_id
		$data = &$GLOBALS['data'];
		$q = 'SELECT `js_cache` FROM `project` WHERE `project`.`id` = ?';
		$result = $data->getOne($q, array($project_id));
		if(DB::isError($result)){
			trigger_error("Could not find project id $project_id during Pool_Project::get_cached_js.", E_USER_WARNING);
			return false;
		} else {
			return $result;
		}	
	}
	
	public static function get_cached_js_short($project_id){
		//Gets the short version of the javascript array for $project_id
		$data = &$GLOBALS['data'];
		$q = 'SELECT `js_cache_short` FROM `project` WHERE `project`.`id` = ?';
		$result = $data->getOne($q, array($project_id));
		if(DB::isError($result)){
			trigger_error("Could not find project id $project_id during Pool_Project::get_cached_js_short.", E_USER_WARNING);
			return false;
		} else {
			return $result;
		}	
	}
	
	public static function get_contributor_list($artifact_type, $artifact_id){
		//gets a list (name, id, roles only) of contributors for the passed artifact and returns it as an array
		$data = &$GLOBALS['data'];
		
		//branch to find contributors for only a single stage or an entire project
		if($artifact_type == 'project'){
			$q = 'SELECT DISTINCT `person`.`id`, `person`.`written`, GROUP_CONCAT(DISTINCT `person_artifact`.`role` SEPARATOR ",") AS roles
					FROM `person`, `person_artifact`, `project_artifact`
					WHERE `project_artifact`.`project_id` = ?
						AND `project_artifact`.`artifact_type` = `person_artifact`.`artifact_type`
						AND `person_artifact`.`artifact_id` = `project_artifact`.`artifact_id` 
						AND `person_artifact`.`person_id` = `person`.`id`
					GROUP BY `person_artifact`.`person_id`';
			$q_arr = array($artifact_id);
		} else {
			$q = 'SELECT `person`.`id`, `person`.`written`, GROUP_CONCAT(DISTINCT `person_artifact`.`role` SEPARATOR ",") AS roles
					FROM `person`, `person_artifact`
					WHERE `person_artifact`.`artifact_id` = ? 
						AND `person_artifact`.`artifact_type` = ?
						AND `person_artifact`.`person_id` = `person`.`id`
					GROUP BY `person_artifact`.`person_id`';
			$q_arr = array($artifact_id, $artifact_type);
		}
		$result = $data->getAll($q, $q_arr);
		if(DB::isError($result)){
			trigger_error("Could not find contributors for $artifact_type with id=$artifact_id in Pool_Project::get_contributor_list.", E_USER_WARNING);
			return false;
		} else {
			return $result;
		}
	}
	
	public static function is_contributor($artifact_type, $artifact_id, $person_id){
		//returns true if the person is a contributor to the artifact
		
		$contrib_list = Pool_Project::get_contributor_list($artifact_type, $artifact_id);
		for($i=0; $i<count($contrib_list); $i++){
			if($contrib_list[$i].id == $person_id) return true;
		}
		return false;
	}
	
	public static function get_reviews($artifact_type, $artifact_id){
		//gets all reviews for a project stage and returns them as an array of Review objects
		$data = &$GLOBALS['data'];
		
		$q = 'SELECT `id` FROM `review` WHERE `artifact_type` = ? AND `artifact_id` = ?';
		$result = $data->getAll($q, array($artifact_type, $artifact_id));
		if(DB::isError($result)){
			trigger_error("Could not get reviews for $artifact_type with id=$artifact_id in Pool_Project::get_reviews.", E_USER_WARNING);
			return false;
		} else {
			$reviews = array();
			foreach($result as $review) $reviews[] = new Review($review['id']);
			return $reviews;
		}
	}
	
	public static function get_aspects($artifact_type, $artifact_id){
		//gets all aspects for the passed artifact and returns them in an array
		$data = &$GLOBALS['data'];
		
		$q = 'SELECT `aspect`, `value` FROM `aspects` WHERE `artifact_type` = ? AND `artifact_id` = ?';
		$result = $data->getAll($q, array($artifact_type, $artifact_id));
		if(DB::isError($result)){
			trigger_error("Could not get aspects for $artifact_type with id=$artifact_id in Pool_Project::get_aspects.", E_USER_WARNING);
			return false;
		} else {
			$aspects = array();
			foreach($result as $aspect) $aspects[$aspect['aspect']] = $aspect['value'];
			return $aspects;
		}		
	}
	
	public static function get_ratings($artifact_type, $artifact_id){
		//gets composite ratings for the passed artifact and returns them in an array
		$data = &$GLOBALS['data'];
		
		$q = 'SELECT `category`, `rating`, `basis` FROM `artifact_ratings` WHERE `artifact_type` = ? AND `artifact_id` = ?';
		$result = $data->getAll($q, array($artifact_type, $artifact_id));
		if(DB::isError($result)){
			trigger_error("Could not get ratings for $artifact_type with id=$artifact_id in Pool_Project::get_ratings.", E_USER_WARNING);
			return false;
		} else {
			$ratings = array();
			foreach($result as $rating) $ratings[$rating['category']] = array('rating' => $rating['rating'], 'basis' => $rating['basis']);
			return $ratings;
		}			
		
	}
	
	public static function new_hr_id($artifact_type, $title){
		//Creates a new hr_id based on the title of a project
		$data = &$GLOBALS['data'];
		$title = str_replace(' ', '', $title);
	
		if(strlen($title) > 18) $title = substr($title, 0, 17);
		if(strlen($artifact_type) > 3) $artifact_type = substr($artifact_type, 0, 3);
		$hr_name = 'pro_'.$artifact_type.'_'.$title;
		$q = 'SELECT count(*) FROM `project` WHERE `hr_id` LIKE ?';
		$result = $data->getOne($q, array($hr_name.'%'));
		if(!self::check_db($result, 'new_hr_id')) return false;
		$result++;
		$hr_num = sprintf("%02d", $result);
		return $hr_name.$hr_num;
	}
}
?>