<?PHP
//Review Class .1
//This implements a class for reviews of all pool objects
//Requires the db_init, Person.class and error_init files

require_once('db_init.php');
require_once('person.class.php');

class Review{
	//Public variables (from db)
	public $id = false;				//ID of the review
	public $artifact_id = false;	//ID of the artifact this review is reviewing
	public $artifact_type = '';		//Type of the artifact (project, intent, etc.)
	public $artifact_title = '';	//Title of the artifact
	public $pool_type = '';			//Type of pool the artifact is a member of
	public $rating = '';			//Overall rating of the review (avg of subratings)
	public $hr_link = '';			//Human readable link ID of the artifact/reviewer
	public $date = '';				//Datetime when the review was last modified
	public $description = '';		//General review text
	public $recorder_id = '';		//person ID of whoever wrote the review
	public $recorder_written = '';	//Name of the reviewer
	public $js_cache = '';

	//metavariables for subratings
	//This will eventually look like $subratings['technical'] = array('rating' => 7, 'description' = 'this is a technical rating);
	public $subratings = array();

	//Protected db object
	//This is filled in the constructor
	protected $data = '';
					
	//Abstract functions				

	//Generic functions
	public function __construct($construct_var){
		//Generic constructor
		//$construct_var can be either a string or an array
		//If $construct_var is a string, it is assumed that you are loading a review from the db
		//If $construct_var is an array, it is assumed you are adding a new review
		
		//Set up the db object
		$this->data = &$GLOBALS['data'];
		
		if(is_array($construct_var)) $this->new_review($construct_var);
		else{
			$this->id = $construct_var;
			$this->load_review();
		}
	}
	
	//new_review is handled in child classes
	public function new_review($form_data){}
	
	public function load_review(){
		//Loads data from the review table
		
		//make sure the id has been set
		if(!$this->check_id('load_review')) return false;
		
		//Get the main review
		$q = 'SELECT * FROM `review` WHERE `id` = ?';
		$result = $this->data->getRow($q, array($this->id));
		if(!$this->check_db($result, 'load_review')) return false;
		foreach($result as $key=>$col) $this->$key = $col;
		//Get the written name of the reviewer
		$this->recorder_written = Pool_Person::get_written($this->recorder_id);
		
		//Get subratings
		$q = 'SELECT * FROM `review_ratings` WHERE `review_id` = ?';
		$result = $this->data->getAll($q, array($this->id));
		if(!$this->check_db($result, 'load_review')) return false;
		foreach($result as $subrating){
			$this->subratings[$subrating['category']] = array('rating' => $subrating['rating'], 'description' => $subrating['description']);
		}
		
		//Get project title
		if($this->artifact_type == 'reference'){
			$reference_title = $this->data->getOne('SELECT `title` FROM `reference` WHERE `id` = ?', array($this->artifact_id));
			if(!$this->check_db($reference_title, 'load_review')) return false;
			$this->artifact_title = $reference_title;
		} else {
			$project_id = Pool_Project::resolve_artifact_id($this->artifact_type, $this->artifact_id);
			$q = 'SELECT `title` FROM `project` WHERE `id` = ?';
			$result = $this->data->getOne($q, array($project_id));
			if(!$this->check_db($result, 'load_review')) return false;
			$this->artifact_title = $result;
		}
	}
	public function write_js(){
		//writes the JS array for this review
		
		require_once('smarty_init.php');
		$smarty = &$GLOBALS['smarty'];
		
		$smarty->assign('review', $this);
		$smarty->left_delimiter = '<{';
		$smarty->right_delimiter = '}>';
		$smarty->display('js_review_array.tpl');
		$smarty->left_delimiter = '{';
		$smarty->right_delimiter = '}';
	}
	
	public function save_js(){
		//writes the JS array for this review
		
		require_once('smarty_init.php');
		$smarty = &$GLOBALS['smarty'];
		
		$smarty->assign('review', $this);
		$smarty->left_delimiter = '<{';
		$smarty->right_delimiter = '}>';
		$js_cache = $smarty->fetch('js_review_array.tpl');
		$smarty->left_delimiter = '{';
		$smarty->right_delimiter = '}';
		
		$q = 'UPDATE `review` SET `js_cache` = ? WHERE `id` = ?';
		$this->data->query($q, array($js_cache, $this->id));
	}	
	
	//Protected functions
	protected function check_id($func){
		//Return true if the id has been set, false and an error otherwise
		if($this->id == false){
			trigger_error("Review->$func failed: No id defined", E_USER_WARNING);
			return false;
		} else return true;
	}
	
	protected function check_db($result, $func){
		//Return true if $result is not DB::Error, false and an error otherwise
		if(DB::isError($result)){
			trigger_error("Review->$func failed:". $this->data->last_query ."\n".$result->getMessage(), E_USER_WARNING);	
			return false;
		} else return true;
	}
}

class Art_Review extends Review{
	public function new_review($form_data){
		//saves a new art review to the database and updates all js_caches
		if(!isset($form_data['artifactId']) || !isset($form_data['artifactType'])) return false;
		
		
		//Get the artifact being reviewed
		$class_name = 'Pool_'.ucfirst($form_data['artifactType']);
		$artifact = new $class_name($form_data['artifactId']);
		if(!$artifact){
			trigger_error("Art_Review->new_review failed: $form_data[artifactType] id $form_data[artifactId] not found.", E_USER_WARNING);
			return false;
		}
		
		//Get the reviewer info
		$reviewer = Pool_Person::cookie_auth();
		if(!$reviewer){
			trigger_error("Art_Review->new_review failed: authentication failed.", E_USER_WARNING);
		}
		
		//Get the project being reviewed
		$project_id = Pool_Project::resolve_artifact_id($form_data['artifactType'], $form_data['artifactId']);
		$project = new Art_Project($project_id);
		if(!$project){
			trigger_error("Art_Review->new_review failed: project with $form_data[artifactType] id $form_data[artifactId] not found.", E_USER_WARNING);
			return false;
		}
		$project->load_all();
		
		//Determine relevent subratings and extract them from the form data
		$var_name = $form_data['artifactType'].'_ratings';
		$categories = $project->$var_name;
		$subratings = array();
		foreach($categories as $category){
			$rating_name = $category.'Rating';
			$description_name = $category.'Description';
			if(isset($form_data[$rating_name]) && $form_data[$rating_name] > 0){
				$subratings[$category] = array('rating' => $form_data[$rating_name], 'description' => $form_data[$description_name]);
			}
		}
		
		//Insert the main review
		$rating_total = 0;
		foreach($subratings as $subrating) $rating_total += $subrating['rating'];
		if(count($subratings) > 0) $rating_total = $rating_total / count($subratings);
		$hr_link = $reviewer['hr_id'].'_'.$artifact->hr_id;
		
		$review_array = array($form_data['artifactId'], $form_data['artifactType'], $rating_total, $hr_link, $form_data['description'], $reviewer['id'], $rating_total, $form_data['description']);
		
		$q = 'INSERT INTO `review` (`artifact_id`, `artifact_type`, `pool_type`, `rating`, `hr_link`, `date`, `description`, `recorder_id`)
				VALUES (?, ?, "art", ?, ?, NOW(), ?, ?) ON DUPLICATE KEY UPDATE `rating` = ?, `date` = NOW(), `description` = ?';
		$result = $this->data->query($q, $review_array);
		$this->check_db($result, 'new_review');
		
		//Get back the id you just put in
		$q = 'SELECT `id` FROM `review` WHERE `hr_link` = ?';
		$review_id = $this->data->getOne($q, array($hr_link));
		
		//Store subratings
		foreach($subratings as $category => $subrating){
			$subrating_link = $hr_link.'_'.$category;
			$subrating_array = array($review_id, $category, $subrating_link, $subrating['rating'], $subrating['description'], $subrating['rating'], $subrating['description']);
			$q = 'INSERT INTO `review_ratings` (`review_id`, `category`, `hr_link`, `rating`, `description`)
					VALUES (?, ?, ?, ?, ?) ON DUPLICATE KEY UPDATE `rating` = ?, `description` = ?';
			$result = $this->data->query($q, $subrating_array);
			$this->check_db($result, 'new_review');
		}
		
		//Load review object
		$this->id = $review_id;
		$this->load_review();
		
		//Update js_caches
		$this->save_js();
		$artifact->save_js();
		//srab writes js caches for the project and its contributors
		$project->reload_all();
		$project->set_rating_and_basis();
		
		//Get the contributor's person object so their js cache can be updated
		$oReviewer = new Pool_Person($reviewer['id']);
		$oReviewer->reload_all();
		$oReviewer->save_js();
		
		return true;
	}
}

class Art_Reference_Review extends Review{
	public function new_review($form_data){
		//saves a new art review to the database and updates all js_caches
		if(!isset($form_data['referenceId'])) return false;
		
		
		//Get the artifact being reviewed
		$reference = new Pool_Reference($form_data['referenceId']);
		if(!$reference){
			trigger_error("Art_Reference_Review->new_review failed: $form_data[referenceType] id $form_data[referenceId] not found.", E_USER_WARNING);
			return false;
		}
		
		//Get the reviewer info
		$reviewer = Pool_Person::cookie_auth();
		if(!$reviewer){
			trigger_error("Art_Reference_Review->new_review failed: authentication failed.", E_USER_WARNING);
		}
				
		//Determine relevent subratings and extract them from the form data
		$art_ini_file = parse_ini_file(POOL_INI_PATH.'art_pool.ini', true);
		$categories = explode(',',$art_ini_file['ratings']['reference']);
		$subratings = array();
		foreach($categories as $category){
			$rating_name = $category.'Rating';
			$description_name = $category.'Description';
			if(isset($form_data[$rating_name]) && $form_data[$rating_name] > 0){
				$subratings[$category] = array('rating' => $form_data[$rating_name], 'description' => $form_data[$description_name]);
			}
		}
		
		//Insert the main review
		$rating_total = 0;
		foreach($subratings as $subrating) $rating_total += $subrating['rating'];
		if(count($subratings) > 0) $rating_total = $rating_total / count($subratings);
		$hr_link = $reviewer['hr_id'].'_'.$reference->hr_id;
		
		$review_array = array($form_data['referenceId'], $form_data['referenceType'], $rating_total, $hr_link, $form_data['description'], $reviewer['id'], $rating_total, $form_data['description']);
		
		$q = 'INSERT INTO `review` (`artifact_id`, `artifact_type`, `pool_type`, `rating`, `hr_link`, `date`, `description`, `recorder_id`)
				VALUES (?, ?, "artreference", ?, ?, NOW(), ?, ?) ON DUPLICATE KEY UPDATE `rating` = ?, `date` = NOW(), `description` = ?';
		$result = $this->data->query($q, $review_array);
		$this->check_db($result, 'new_review');
		
		//Get back the id you just put in
		$q = 'SELECT `id` FROM `review` WHERE `hr_link` = ?';
		$review_id = $this->data->getOne($q, array($hr_link));
		
		//Store subratings
		foreach($subratings as $category => $subrating){
			$subrating_link = $hr_link.'_'.$category;
			$subrating_array = array($review_id, $category, $subrating_link, $subrating['rating'], $subrating['description'], $subrating['rating'], $subrating['description']);
			$q = 'INSERT INTO `review_ratings` (`review_id`, `category`, `hr_link`, `rating`, `description`)
					VALUES (?, ?, ?, ?, ?) ON DUPLICATE KEY UPDATE `rating` = ?, `description` = ?';
			$result = $this->data->query($q, $subrating_array);
			$this->check_db($result, 'new_review');
		}
		
		//Load review object
		$this->id = $review_id;
		$this->load_review();
		
		//Update js_caches
		$this->save_js();
		$reference->reload_all();
		$reference->save_js();
		//srab writes js caches for the project and its contributors
		$reference->set_rating_and_basis();
		
		//Get the contributor's person object so their js cache can be updated
		$oReviewer = new Pool_Person($reviewer['id']);
		$oReviewer->reload_all();
		$oReviewer->save_js();
		
		return true;
	}
}

class Code_Review extends Review{
	public function new_review($form_data){
		//saves a new code review to the database and updates all js_caches
		if(!isset($form_data['artifactId']) || !isset($form_data['artifactType'])) return false;
		
		
		//Get the artifact being reviewed
		$class_name = 'Pool_'.ucfirst($form_data['artifactType']);
		$artifact = new $class_name($form_data['artifactId']);
		if(!$artifact){
			trigger_error("Code_Review->new_review failed: $form_data[artifactType] id $form_data[artifactId] not found.", E_USER_WARNING);
			return false;
		}
		
		//Get the reviewer info
		$reviewer = Pool_Person::cookie_auth();
		if(!$reviewer){
			trigger_error("Code_Review->new_review failed: authentication failed.", E_USER_WARNING);
		}
		
		//Get the project being reviewed
		$project_id = Pool_Project::resolve_artifact_id($form_data['artifactType'], $form_data['artifactId']);
		$project = new Code_Project($project_id);
		if(!$project){
			trigger_error("Code_Review->new_review failed: project with $form_data[artifactType] id $form_data[artifactId] not found.", E_USER_WARNING);
			return false;
		}
		$project->load_all();
		
		//Determine relevent subratings and extract them from the form data
		$var_name = $form_data['artifactType'].'_ratings';
		$categories = $project->$var_name;
		$subratings = array();
		foreach($categories as $category){
			$rating_name = $category.'Rating';
			$description_name = $category.'Description';
			if(isset($form_data[$rating_name]) && $form_data[$rating_name] > 0){
				$subratings[$category] = array('rating' => $form_data[$rating_name], 'description' => $form_data[$description_name]);
			}
		}
		
		//Insert the main review
		$rating_total = 0;
		foreach($subratings as $subrating) $rating_total += $subrating['rating'];
		if(count($subratings) > 0) $rating_total = $rating_total / count($subratings);
		$hr_link = $reviewer['hr_id'].'_'.$artifact->hr_id;
		
		$review_array = array($form_data['artifactId'], $form_data['artifactType'], $rating_total, $hr_link, $form_data['description'], $reviewer['id'], $rating_total, $form_data['description']);
		
		$q = 'INSERT INTO `review` (`artifact_id`, `artifact_type`, `pool_type`, `rating`, `hr_link`, `date`, `description`, `recorder_id`)
				VALUES (?, ?, "code", ?, ?, NOW(), ?, ?) ON DUPLICATE KEY UPDATE `rating` = ?, `date` = NOW(), `description` = ?';
		$result = $this->data->query($q, $review_array);
		$this->check_db($result, 'new_review');
		
		//Get back the id you just put in
		$q = 'SELECT `id` FROM `review` WHERE `hr_link` = ?';
		$review_id = $this->data->getOne($q, array($hr_link));
		
		//Store subratings
		foreach($subratings as $category => $subrating){
			$subrating_link = $hr_link.'_'.$category;
			$subrating_array = array($review_id, $category, $subrating_link, $subrating['rating'], $subrating['description'], $subrating['rating'], $subrating['description']);
			$q = 'INSERT INTO `review_ratings` (`review_id`, `category`, `hr_link`, `rating`, `description`)
					VALUES (?, ?, ?, ?, ?) ON DUPLICATE KEY UPDATE `rating` = ?, `description` = ?';
			$result = $this->data->query($q, $subrating_array);
			$this->check_db($result, 'new_review');
		}
		
		//Load review object
		$this->id = $review_id;
		$this->load_review();
		
		//Update js_caches
		$this->save_js();
		$artifact->save_js();
		//srab writes js caches for the project and its contributors
		$project->reload_all();
		$project->set_rating_and_basis();
		
		//Get the contributor's person object so their js cache can be updated
		$oReviewer = new Pool_Person($reviewer['id']);
		$oReviewer->reload_all();
		$oReviewer->save_js();
		
		return true;
	}
}

class Code_Reference_Review extends Review{
	public function new_review($form_data){
		//saves a new code reference review to the database and updates all js_caches
		if(!isset($form_data['referenceId'])) return false;
		
		
		//Get the artifact being reviewed
		$reference = new Pool_Reference($form_data['referenceId']);
		if(!$reference){
			trigger_error("Code_Reference_Review->new_review failed: $form_data[referenceType] id $form_data[referenceId] not found.", E_USER_WARNING);
			return false;
		}
		
		//Get the reviewer info
		$reviewer = Pool_Person::cookie_auth();
		if(!$reviewer){
			trigger_error("Code_Reference_Review->new_review failed: authentication failed.", E_USER_WARNING);
		}
				
		//Determine relevent subratings and extract them from the form data
		$art_ini_file = parse_ini_file(POOL_INI_PATH.'code_pool.ini', true);
		$categories = explode(',',$art_ini_file['ratings']['reference']);
		$subratings = array();
		foreach($categories as $category){
			$rating_name = $category.'Rating';
			$description_name = $category.'Description';
			if(isset($form_data[$rating_name]) && $form_data[$rating_name] > 0){
				$subratings[$category] = array('rating' => $form_data[$rating_name], 'description' => $form_data[$description_name]);
			}
		}
		
		//Insert the main review
		$rating_total = 0;
		foreach($subratings as $subrating) $rating_total += $subrating['rating'];
		if(count($subratings) > 0) $rating_total = $rating_total / count($subratings);
		$hr_link = $reviewer['hr_id'].'_'.$reference->hr_id;
		
		$review_array = array($form_data['referenceId'], $form_data['referenceType'], $rating_total, $hr_link, $form_data['description'], $reviewer['id'], $rating_total, $form_data['description']);
		
		$q = 'INSERT INTO `review` (`artifact_id`, `artifact_type`, `pool_type`, `rating`, `hr_link`, `date`, `description`, `recorder_id`)
				VALUES (?, ?, "codereference", ?, ?, NOW(), ?, ?) ON DUPLICATE KEY UPDATE `rating` = ?, `date` = NOW(), `description` = ?';
		$result = $this->data->query($q, $review_array);
		$this->check_db($result, 'new_review');
		
		//Get back the id you just put in
		$q = 'SELECT `id` FROM `review` WHERE `hr_link` = ?';
		$review_id = $this->data->getOne($q, array($hr_link));
		
		//Store subratings
		foreach($subratings as $category => $subrating){
			$subrating_link = $hr_link.'_'.$category;
			$subrating_array = array($review_id, $category, $subrating_link, $subrating['rating'], $subrating['description'], $subrating['rating'], $subrating['description']);
			$q = 'INSERT INTO `review_ratings` (`review_id`, `category`, `hr_link`, `rating`, `description`)
					VALUES (?, ?, ?, ?, ?) ON DUPLICATE KEY UPDATE `rating` = ?, `description` = ?';
			$result = $this->data->query($q, $subrating_array);
			$this->check_db($result, 'new_review');
		}
		
		//Load review object
		$this->id = $review_id;
		$this->load_review();
		
		//Update js_caches
		$this->save_js();
		$reference->reload_all();
		$reference->save_js();
		//srab writes js caches for the project and its contributors
		$reference->set_rating_and_basis();
		
		//Get the contributor's person object so their js cache can be updated
		$oReviewer = new Pool_Person($reviewer['id']);
		$oReviewer->reload_all();
		$oReviewer->save_js();
		
		return true;
	}
}


?>